<html>
  <head>
    <title>Login</title>
  </head>
  <body>
    <form>
      Username:<input type="text" name="username"/><br/>
      Password:<input type="password" name="password"/><br/>
      <button>Login</button>
    </form>
  </body>
</html>
